@@ -4,9 +4,9 @@
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -17,17 +17,15 @@ package ghidra.app.plugin.core.debug.gui.model;
 
 import java.awt.Color;
 import java.util.*;
-import java.util.stream.Collectors;
-import java.util.stream.Stream;
+import java.util.stream.*;
+
+import org.apache.commons.lang3.ArrayUtils;
 
 import docking.widgets.table.DynamicTableColumn;
 import docking.widgets.table.RangeCursorTableHeaderRenderer.SeekListener;
 import docking.widgets.table.TableColumnDescriptor;
 import ghidra.app.plugin.core.debug.gui.model.ObjectTableModel.ValueRow;
 import ghidra.app.plugin.core.debug.gui.model.columns.*;
-import ghidra.dbg.target.schema.SchemaContext;
-import ghidra.dbg.target.schema.TargetObjectSchema;
-import ghidra.dbg.target.schema.TargetObjectSchema.AttributeSchema;
 import ghidra.docking.settings.Settings;
 import ghidra.framework.plugintool.Plugin;
 import ghidra.framework.plugintool.ServiceProvider;
@@ -37,7 +35,12 @@ import ghidra.trace.model.Lifespan.*;
 import ghidra.trace.model.Trace;
 import ghidra.trace.model.target.TraceObject;
 import ghidra.trace.model.target.TraceObjectValue;
+import ghidra.trace.model.target.schema.SchemaContext;
+import ghidra.trace.model.target.schema.TraceObjectSchema;
+import ghidra.trace.model.target.schema.TraceObjectSchema.AttributeSchema;
 import ghidra.util.HTMLUtilities;
+import ghidra.util.NumericUtilities;
+import ghidra.util.datastruct.ListenerSet;
 
 public class ObjectTableModel extends AbstractQueryTableModel<ValueRow> {
 
@@ -61,7 +64,36 @@ public class ObjectTableModel extends AbstractQueryTableModel<ValueRow> {
 
 		default public String getDisplay() {
 			T value = getValue();
-			return value == null ? "" : value.toString();
+			if (value == null) {
+				return "";
+			}
+			if (value instanceof boolean[] bools) {
+				return Stream.of(ArrayUtils.toObject(bools))
+						.map(b -> b ? "T" : "F")
+						.collect(Collectors.joining(":"));
+			}
+			if (value instanceof byte[] bytes) {
+				return NumericUtilities.convertBytesToString(bytes, ":");
+			}
+			if (value instanceof char[] chars) {
+				return new String(chars);
+			}
+			if (value instanceof short[] shorts) {
+				return Stream.of(ArrayUtils.toObject(shorts))
+						.map(s -> "%04x".formatted(s))
+						.collect(Collectors.joining(":"));
+			}
+			if (value instanceof int[] ints) {
+				return IntStream.of(ints)
+						.mapToObj(i -> "%08x".formatted(i))
+						.collect(Collectors.joining(":"));
+			}
+			if (value instanceof long[] longs) {
+				return LongStream.of(longs)
+						.mapToObj(l -> "%016x".formatted(l))
+						.collect(Collectors.joining(":"));
+			}
+			return value.toString();
 		}
 
 		default public String getHtmlDisplay() {
@@ -134,7 +166,7 @@ public class ObjectTableModel extends AbstractQueryTableModel<ValueRow> {
 		}
 	}
 
-	public record ValueAttribute<T> (ValueRow row, String name, Class<T> type)
+	public record ValueAttribute<T>(ValueRow row, String name, Class<T> type)
 			implements ValueProperty<T> {
 		public TraceObjectValue getEntry() {
 			return row.getAttributeEntry(name);
@@ -290,7 +322,7 @@ public class ObjectTableModel extends AbstractQueryTableModel<ValueRow> {
 		@Override
 		public String getHtmlDisplay() {
 			return "<html>" +
-				HTMLUtilities.escapeHTML(display.getPrimitiveValueDisplay(value.getValue()));
+				HTMLUtilities.escapeHTML(display.getPrimitiveValueDisplay(value.getValue()), true);
 		}
 
 		@Override
@@ -412,134 +444,100 @@ public class ObjectTableModel extends AbstractQueryTableModel<ValueRow> {
 				AttributeSchema attributeSchema) {
 			String name = attributeSchema.getName();
 			Class<?> type = computeAttributeType(ctx, attributeSchema);
-			return new AutoAttributeColumn<>(name, type);
+			return new AutoAttributeColumn<>(name, type, attributeSchema.isHidden(name));
 		}
 
-		public AutoAttributeColumn(String attributeName, Class<T> attributeType) {
+		final boolean hidden;
+
+		public AutoAttributeColumn(String attributeName, Class<T> attributeType, boolean hidden) {
 			super(attributeName, attributeType);
+			this.hidden = hidden;
+		}
+
+		public boolean isHidden() {
+			return hidden;
 		}
 	}
 
-	// TODO: Save and restore these between sessions, esp., their settings
+	private TraceValueLifePlotColumn plotColumn;
+	private ListenerSet<SeekListener> seekListeners;
+	private SeekListener mySeekListener;
 	private Map<ColKey, TraceValueObjectAttributeColumn<?>> columnCache = new HashMap<>();
 
 	protected ObjectTableModel(Plugin plugin) {
 		super("Object Model", plugin);
 	}
 
+	protected TraceValueLifePlotColumn newPlotColumn() {
+		return new TraceValueLifePlotColumn();
+	}
+
+	protected TraceValueLifePlotColumn getPlotColumn() {
+		if (plotColumn == null) {
+			plotColumn = newPlotColumn();
+			plotColumn.setFullRange(computeFullRange());
+			getSeekListeners();
+			plotColumn.addSeekListener(mySeekListener);
+		}
+		return plotColumn;
+	}
+
+	protected ListenerSet<SeekListener> getSeekListeners() {
+		if (seekListeners == null) {
+			seekListeners = new ListenerSet<>(SeekListener.class, true);
+			// Can't use the proxy directly, as Set will invoke hashCode/equals
+			mySeekListener = seekListeners.invoke()::accept;
+		}
+		return seekListeners;
+	}
+
 	@Override
 	protected void traceChanged() {
-		reloadAttributeColumns();
+		reloadColumns();
+		fireTableStructureChanged();
 		updateTimelineMax();
 		super.traceChanged();
 	}
 
 	@Override
 	protected void queryChanged() {
-		reloadAttributeColumns();
+		reloadColumns();
+		fireTableStructureChanged();
 		super.queryChanged();
 	}
 
-	@Override
-	protected void showHiddenChanged() {
-		reloadAttributeColumns();
-		super.showHiddenChanged();
-	}
-
 	@Override
 	protected void maxSnapChanged() {
 		updateTimelineMax();
 		refresh();
 	}
 
-	protected void updateTimelineMax() {
+	protected Lifespan computeFullRange() {
 		Long max = getTrace() == null ? null : getTrace().getTimeManager().getMaxSnap();
-		Lifespan fullRange = Lifespan.span(0L, max == null ? 1 : max + 1);
-		int count = getColumnCount();
-		for (int i = 0; i < count; i++) {
-			DynamicTableColumn<ValueRow, ?, ?> column = getColumn(i);
-			if (column instanceof TraceValueLifePlotColumn plotCol) {
-				plotCol.setFullRange(fullRange);
-			}
-		}
+		return Lifespan.span(0L, max == null ? 1 : max + 1);
 	}
 
-	protected List<AttributeSchema> computeAttributeSchemas() {
+	protected void updateTimelineMax() {
+		Lifespan fullRange = computeFullRange();
+		getPlotColumn().setFullRange(fullRange);
+	}
+
+	protected List<AttributeSchema> computeAttributes() {
 		Trace trace = getTrace();
 		ModelQuery query = getQuery();
 		if (trace == null || query == null) {
 			return List.of();
 		}
-		TargetObjectSchema rootSchema = trace.getObjectManager().getRootSchema();
+		TraceObjectSchema rootSchema = trace.getObjectManager().getRootSchema();
 		if (rootSchema == null) {
 			return List.of();
 		}
 		SchemaContext ctx = rootSchema.getContext();
 		return query.computeAttributes(trace)
-				.filter(a -> isShowHidden() || !a.isHidden())
 				.filter(a -> !ctx.getSchema(a.getSchema()).isCanonicalContainer())
 				.collect(Collectors.toList());
 	}
 
-	protected void reloadAttributeColumns() {
-		List<AttributeSchema> attributes;
-		Trace trace = getTrace();
-		ModelQuery query = getQuery();
-		if (trace == null || query == null || trace.getObjectManager().getRootSchema() == null) {
-			attributes = List.of();
-		}
-		else {
-			SchemaContext ctx = trace.getObjectManager().getRootSchema().getContext();
-			attributes = query.computeAttributes(trace)
-					.filter(a -> !ctx.getSchema(a.getSchema()).isCanonicalContainer())
-					.collect(Collectors.toList());
-		}
-		resyncAttributeColumns(attributes);
-	}
-
-	protected Set<DynamicTableColumn<ValueRow, ?, ?>> computeAttributeColumns(
-			Collection<AttributeSchema> attributes) {
-		if (attributes == null) {
-			return Set.of();
-		}
-		Trace trace = getTrace();
-		if (trace == null) {
-			return Set.of();
-		}
-		TargetObjectSchema rootSchema = trace.getObjectManager().getRootSchema();
-		if (rootSchema == null) {
-			return Set.of();
-		}
-		SchemaContext ctx = rootSchema.getContext();
-		return attributes.stream()
-				.map(as -> columnCache.computeIfAbsent(ColKey.fromSchema(ctx, as),
-					ck -> AutoAttributeColumn.fromSchema(ctx, as)))
-				.collect(Collectors.toSet());
-	}
-
-	protected void resyncAttributeColumns(Collection<AttributeSchema> attributes) {
-		Map<Boolean, List<AttributeSchema>> byVisible = attributes == null ? Map.of()
-				: attributes.stream()
-						.collect(Collectors.groupingBy(a -> !a.isHidden() || isShowHidden()));
-		Set<DynamicTableColumn<ValueRow, ?, ?>> visibleColumns =
-			new HashSet<>(computeAttributeColumns(byVisible.get(true)));
-		Set<DynamicTableColumn<ValueRow, ?, ?>> hiddenColumns =
-			new HashSet<>(computeAttributeColumns(byVisible.get(false)));
-		Set<DynamicTableColumn<ValueRow, ?, ?>> toRemove = new HashSet<>();
-		for (int i = 0; i < getColumnCount(); i++) {
-			DynamicTableColumn<ValueRow, ?, ?> exists = getColumn(i);
-			if (!(exists instanceof AutoAttributeColumn)) {
-				continue;
-			}
-			if (!visibleColumns.remove(exists) && !hiddenColumns.remove(exists)) {
-				toRemove.add(exists);
-			}
-		}
-		removeTableColumns(toRemove);
-		addTableColumns(visibleColumns, true);
-		addTableColumns(hiddenColumns, false);
-	}
-
 	@Override
 	protected Stream<ValueRow> streamRows(Trace trace, ModelQuery query, Lifespan span) {
 		return distinctCanonical(query.streamValues(trace, span)
@@ -550,13 +548,40 @@ public class ObjectTableModel extends AbstractQueryTableModel<ValueRow> {
 	@Override
 	protected TableColumnDescriptor<ValueRow> createTableColumnDescriptor() {
 		TableColumnDescriptor<ValueRow> descriptor = new TableColumnDescriptor<>();
-		descriptor.addVisibleColumn(new TraceValueKeyColumn());
+		descriptor.addVisibleColumn(new TraceValueKeyColumn(), 1, true);
 		descriptor.addVisibleColumn(new TraceValueValColumn());
-		descriptor.addVisibleColumn(new TraceValueLifeColumn());
-		descriptor.addHiddenColumn(new TraceValueLifePlotColumn());
+		descriptor.addVisibleColumn(new TraceValueLifeColumn(), 2, true);
+		descriptor.addHiddenColumn(getPlotColumn());
+
+		appendAttributeColumns(descriptor);
+
 		return descriptor;
 	}
 
+	protected void appendAttributeColumns(TableColumnDescriptor<ValueRow> descriptor) {
+		Trace trace = getTrace();
+		if (trace == null) {
+			return;
+		}
+		TraceObjectSchema rootSchema = trace.getObjectManager().getRootSchema();
+		if (rootSchema == null) {
+			return;
+		}
+		SchemaContext ctx = rootSchema.getContext();
+		List<AttributeSchema> attributes = computeAttributes();
+		for (AttributeSchema as : attributes) {
+			TraceValueObjectAttributeColumn<?> column =
+				columnCache.computeIfAbsent(ColKey.fromSchema(ctx, as),
+					ck -> AutoAttributeColumn.fromSchema(ctx, as));
+			if (as.isHidden(as.getName())) {
+				descriptor.addHiddenColumn(column);
+			}
+			else {
+				descriptor.addVisibleColumn(column);
+			}
+		}
+	}
+
 	@Override
 	public ValueRow findTraceObject(TraceObject object) {
 		for (ValueRow row : getModelData()) {
@@ -624,25 +649,12 @@ public class ObjectTableModel extends AbstractQueryTableModel<ValueRow> {
 	@Override
 	protected void snapChanged() {
 		super.snapChanged();
-		long snap = getSnap();
-		int count = getColumnCount();
-		for (int i = 0; i < count; i++) {
-			DynamicTableColumn<ValueRow, ?, ?> column = getColumn(i);
-			if (column instanceof TraceValueLifePlotColumn plotCol) {
-				plotCol.setSnap(snap);
-			}
-		}
+		getPlotColumn().setSnap(getSnap());
 	}
 
 	@Override
 	public void addSeekListener(SeekListener listener) {
-		int count = getColumnCount();
-		for (int i = 0; i < count; i++) {
-			DynamicTableColumn<ValueRow, ?, ?> column = getColumn(i);
-			if (column instanceof TraceValueLifePlotColumn plotCol) {
-				plotCol.addSeekListener(listener);
-			}
-		}
+		getSeekListeners().add(listener);
 	}
 
 	@Override
@@ -714,4 +726,13 @@ public class ObjectTableModel extends AbstractQueryTableModel<ValueRow> {
 			ServiceProvider serviceProvider) {
 		editable.setValue(t, (COLUMN_TYPE) aValue, settings, dataSource, serviceProvider);
 	}
+
+	@Override
+	public boolean isVisibleByDefault(int modelIndex) {
+		DynamicTableColumn<ValueRow, ?, ?> column = tableColumns.get(modelIndex);
+		if (column instanceof AutoAttributeColumn<?> && isShowHidden()) {
+			return true;
+		}
+		return super.isVisibleByDefault(modelIndex);
+	}
 }
