@@ -58,6 +58,11 @@ import ghidra.trace.model.breakpoint.TraceBreakpointKind;
 import ghidra.trace.model.guest.TracePlatform;
 import ghidra.trace.model.memory.TraceMemoryManager;
 import ghidra.trace.model.memory.TraceMemorySpace;
+import ghidra.trace.model.stack.TraceObjectStack;
+import ghidra.trace.model.stack.TraceObjectStackFrame;
+import ghidra.trace.model.target.path.KeyPath;
+import ghidra.trace.model.target.path.PathFilter;
+import ghidra.trace.model.target.schema.TraceObjectSchema;
 import ghidra.trace.model.thread.TraceThread;
 import ghidra.trace.model.time.TraceSnapshot;
 import ghidra.trace.model.time.schedule.Scheduler;
@@ -976,4 +981,15 @@ public class DebuggerEmulationServiceTest extends AbstractGhidraHeadedDebuggerTe
 			PcodeThread<byte[]> newEmuThread = emulator.newThread(newTraceThread.getPath());
 		}
 	}
+
+	@Test
+	public void testEmuSchemaHasWorkingStackFrames() throws Exception {
+		TraceObjectSchema rootSchema = ProgramEmulationUtils.EMU_SESSION_SCHEMA;
+		TraceObjectSchema threadSchema = rootSchema.getSuccessorSchema(KeyPath.parse("Threads[1]"));
+		KeyPath found = threadSchema.searchForCanonicalContainer(TraceObjectStackFrame.class);
+		assertEquals(KeyPath.parse("Stack"), found);
+
+		PathFilter stackFilter = threadSchema.searchFor(TraceObjectStack.class, false);
+		assertNotNull("Non-unique Stack", stackFilter.getSingletonPath());
+	}
 }
