@@ -33,7 +33,7 @@ import ghidra.util.exception.*;
 import ghidra.util.task.TaskLauncher;
 import ghidra.util.task.TaskMonitor;
 
-class StructureEditorModel extends CompEditorModel {
+class StructureEditorModel extends CompEditorModel<Structure> {
 
 	private static final long serialVersionUID = 1L;
 	private static final int OFFSET = 0;
@@ -184,14 +184,13 @@ class StructureEditorModel extends CompEditorModel {
 		if (rowIndex < 0 || rowIndex == numComponents) {
 			return null;
 		}
-		Structure viewStruct = (Structure) viewComposite;
 		if (rowIndex > numComponents) {
 			return null;
 		}
 		if (isShowingUndefinedBytes()) {
 			return viewComposite.getComponent(rowIndex);
 		}
-		DataTypeComponent[] definedComponents = viewStruct.getDefinedComponents();
+		DataTypeComponent[] definedComponents = viewComposite.getDefinedComponents();
 		return definedComponents[rowIndex];
 	}
 
@@ -215,8 +214,7 @@ class StructureEditorModel extends CompEditorModel {
 		}
 
 		viewDTM.withTransaction("Set Size", () -> {
-			Structure structure = (Structure) viewComposite;
-			structure.setLength(size);
+			viewComposite.setLength(size);
 		});
 		notifyCompositeChanged();
 	}
@@ -278,7 +276,7 @@ class StructureEditorModel extends CompEditorModel {
 				}
 				boolean isSelected = selection.containsEntirely(BigInteger.valueOf(indices[i]));
 				int numBytes = comp.getLength();
-				((Structure) viewComposite).clearComponent(indices[i]);
+				viewComposite.clearComponent(indices[i]);
 
 				// Adjust the selection due to the clear.
 				adjustSelection(indices[i] + 1, numBytes - 1);
@@ -291,7 +289,6 @@ class StructureEditorModel extends CompEditorModel {
 				}
 			}
 		});
-		componentEdited();
 	}
 
 	@Override
@@ -309,7 +306,7 @@ class StructureEditorModel extends CompEditorModel {
 
 	private int[] convertRowsToOrdinals(int[] rows) {
 		int[] ordinals = new int[rows.length];
-		DataTypeComponent[] definedComponents = ((Structure) viewComposite).getDefinedComponents();
+		DataTypeComponent[] definedComponents = viewComposite.getDefinedComponents();
 		for (int i = rows.length - 1; i >= 0; i--) {
 			ordinals[i] = definedComponents[rows[i]].getOrdinal();
 		}
@@ -328,7 +325,7 @@ class StructureEditorModel extends CompEditorModel {
 		if (isShowingUndefinedBytes()) {
 			return rowIndex;
 		}
-		DataTypeComponent[] definedComponents = ((Structure) viewComposite).getDefinedComponents();
+		DataTypeComponent[] definedComponents = viewComposite.getDefinedComponents();
 		return definedComponents[rowIndex].getOrdinal();
 	}
 
@@ -366,7 +363,6 @@ class StructureEditorModel extends CompEditorModel {
 		// Ensure that last added component is selected to allow for repeated duplication
 		setSelection(new int[] { index + multiple });
 
-		componentEdited();
 		lastNumDuplicates = multiple;
 	}
 
@@ -485,7 +481,6 @@ class StructureEditorModel extends CompEditorModel {
 		int newIndex = startRowIndex - 1;
 		moved = shiftComponentsUp(startRowIndex, endRowIndex);
 		if (moved) {
-			componentEdited();
 			FieldSelection tmpFieldSelection = new FieldSelection();
 			tmpFieldSelection.addRange(newIndex, newIndex + numSelected);
 			setSelection(tmpFieldSelection);
@@ -509,7 +504,6 @@ class StructureEditorModel extends CompEditorModel {
 		int newIndex = startIndex + 1;
 		moved = shiftComponentsDown(startIndex, endIndex);
 		if (moved) {
-			componentEdited();
 			FieldSelection tmpFieldSelection = new FieldSelection();
 			tmpFieldSelection.addRange(newIndex, newIndex + numSelected);
 			setSelection(tmpFieldSelection);
@@ -737,16 +731,16 @@ class StructureEditorModel extends CompEditorModel {
 		catch (InvalidDataTypeException e) {
 			return false;
 		}
-		
+
 		if (isPackingEnabled() || isAtEnd(rowIndex)) {
 			return true;
 		}
-		
+
 		int undefSize = getNumUndefinedBytesAfter(dtc);
 		if (undefSize < 0) {
 			return true;
 		}
-		
+
 		int numAvailable = dtc.getLength() + undefSize;
 		return dataType.getLength() <= numAvailable;
 	}
@@ -806,7 +800,7 @@ class StructureEditorModel extends CompEditorModel {
 		if (isPackingEnabled() || isAtEnd(currentIndex)) {
 			return Integer.MAX_VALUE;
 		}
-		
+
 		// Can only replace with what fits unless at last component or empty last line.
 		DataTypeComponent comp = getComponent(currentIndex);
 		int numComponents = getNumComponents();
@@ -820,9 +814,9 @@ class StructureEditorModel extends CompEditorModel {
 		// Otherwise, get size of component and number of Undefined bytes after it.
 		FieldRange currentRange = getSelectedRangeContaining(currentIndex);
 		boolean isOneComponent =
-				(currentRange == null) || (currentRange.getStart().getIndex().intValue() +
-					1 == currentRange.getEnd().getIndex().intValue());
-		
+			(currentRange == null) || (currentRange.getStart().getIndex().intValue() +
+				1 == currentRange.getEnd().getIndex().intValue());
+
 		if (isOneComponent) {
 			return comp.getLength() + getNumUndefinedBytesAfter(comp);
 		}
@@ -860,12 +854,12 @@ class StructureEditorModel extends CompEditorModel {
 			return viewDTM.withTransaction("Insert Component", () -> {
 				DataTypeComponent dtc;
 				if (isPackingEnabled() || !(dataType instanceof BitFieldDataType)) {
-					dtc = ((Structure) viewComposite).insert(rowIndex, dataType, length, name,
+					dtc = viewComposite.insert(rowIndex, dataType, length, name,
 						comment);
 				}
 				else {
 					BitFieldDataType bitfield = (BitFieldDataType) dataType;
-					dtc = ((Structure) viewComposite).insertBitField(rowIndex, length,
+					dtc = viewComposite.insertBitField(rowIndex, length,
 						bitfield.getBitOffset(), bitfield.getBaseDataType(),
 						bitfield.getDeclaredBitSize(), name, comment);
 				}
@@ -927,7 +921,7 @@ class StructureEditorModel extends CompEditorModel {
 			if (!isPackingEnabled() && !isAtEnd(rowIndex)) {
 				int origLen = getComponent(rowIndex).getLength();
 				dtc = viewDTM.withTransaction("Replace Component", () -> {
-					return ((Structure) viewComposite).replace(componentOrdinal, dataType, length,
+					return viewComposite.replace(componentOrdinal, dataType, length,
 						name, comment);
 				});
 				diffLen = origLen - dtc.getLength();
@@ -948,16 +942,16 @@ class StructureEditorModel extends CompEditorModel {
 			}
 			else {
 				dtc = viewDTM.withTransaction("Replace Component", () -> {
-					Structure struct = (Structure) viewComposite;
 					DataTypeComponent comp = getComponent(rowIndex);
 					if (!isPackingEnabled()) {
 						// We are at end with packing disabled - grow structure if needed
 						int avail = comp.getLength() + getNumUndefinedBytesAfter(comp);
 						if (length > avail) {
-							struct.growStructure(length - avail);
+							viewComposite.growStructure(length - avail);
 						}
 					}
-					return ((Structure) viewComposite).replace(componentOrdinal, dataType, length, name, comment);
+					return viewComposite.replace(componentOrdinal, dataType, length,
+						name, comment);
 				});
 			}
 			return dtc;
@@ -1051,9 +1045,8 @@ class StructureEditorModel extends CompEditorModel {
 
 	@Override
 	protected void replaceOriginalComponents() {
-		Structure dt = (Structure) getOriginalComposite();
-		if (dt != null) {
-			dt.replaceWith(viewComposite);
+		if (originalComposite != null) {
+			originalComposite.replaceWith(viewComposite);
 		}
 		else {
 			throw new RuntimeException("ERROR: Couldn't replace structure components in " +
@@ -1061,33 +1054,6 @@ class StructureEditorModel extends CompEditorModel {
 		}
 	}
 
-	@Override
-	void removeDtFromComponents(Composite comp) {
-		DataType newDt = viewDTM.getDataType(comp.getDataTypePath());
-		if (newDt == null) {
-			return;
-		}
-		boolean clearedComponents = viewDTM.withTransaction("Remove Components", () -> {
-			boolean cleared = false;
-			int num = getNumComponents();
-			for (int i = num - 1; i >= 0; i--) {
-				DataTypeComponent dtc = getComponent(i);
-				DataType dt = dtc.getDataType();
-				if (dt instanceof Composite) {
-					Composite dtcComp = (Composite) dt;
-					if (dtcComp.isPartOf(newDt)) {
-						clearComponents(new int[] { i });
-						cleared = true;
-					}
-				}
-			}
-			return cleared;
-		});
-		if (clearedComponents) {
-			setStatus("Components containing " + comp.getDisplayName() + " were cleared.", true);
-		}
-	}
-
 	@Override
 	public boolean isShowingUndefinedBytes() {
 		return !viewComposite.isPackingEnabled();
@@ -1119,17 +1085,8 @@ class StructureEditorModel extends CompEditorModel {
 		}
 
 		DataTypeManager originalDtm = getOriginalDataTypeManager();
-		String baseName = "struct";
 		CategoryPath originalCategoryPath = getOriginalCategoryPath();
-		String uniqueName = viewDTM.getUniqueName(originalCategoryPath, baseName);
-		DataType conflictingDt = originalDtm.getDataType(originalCategoryPath, uniqueName);
-		while (conflictingDt != null) {
-			// pull the data type into the view data type manager with the conflicting name.
-			viewDTM.resolve(conflictingDt, DataTypeConflictHandler.DEFAULT_HANDLER);
-			// Try to get another unique name.
-			uniqueName = viewDTM.getUniqueName(originalCategoryPath, baseName);
-			conflictingDt = originalDtm.getDataType(originalCategoryPath, uniqueName);
-		}
+		String uniqueName = getUniqueDataTypeName(originalCategoryPath, "struct");
 
 		String specifiedName =
 			showNameDialog(uniqueName, originalCategoryPath, viewComposite.getName(), originalDtm);
@@ -1148,6 +1105,39 @@ class StructureEditorModel extends CompEditorModel {
 		});
 	}
 
+	/**
+	 * Get a unique datatype name.  Method based on the logic used by 
+	 * {@link DataTypeManager#getUniqueName(CategoryPath, String)} but 
+	 * checks both the original and view DTMs for uniqueness.
+	 * @param path category path
+	 * @param baseName datatype name
+	 * @return unique datatype name
+	 */
+	private String getUniqueDataTypeName(CategoryPath path, String baseName) {
+		int pos = baseName.lastIndexOf('_');
+		int oneUpNumber = 0;
+		String name = baseName;
+		if (pos > 0) {
+			String numString = baseName.substring(pos + 1);
+			try {
+				oneUpNumber = Integer.parseInt(numString);
+				name = baseName;
+				baseName = baseName.substring(0, pos);
+			}
+			catch (NumberFormatException e) {
+				// the number will get updated below
+			}
+		}
+		DataTypeManager originalDtm = getOriginalDataTypeManager();
+		// Pick a name not used within either original or view DTM
+		while (originalDtm.getDataType(path, name) != null ||
+			viewDTM.getDataType(path, name) != null) {
+			++oneUpNumber;
+			name = baseName + "_" + oneUpNumber;
+		}
+		return name;
+	}
+
 	private void doCreateInternalStructure(DataTypeManager dtm, CategoryPath categoryPath,
 			String name, TaskMonitor monitor) throws InvalidDataTypeException, UsrException {
 
@@ -1242,8 +1232,9 @@ class StructureEditorModel extends CompEditorModel {
 				dialog.setStatusText("The name cannot match the external structure name.");
 				return false;
 			}
-			DataTypeManager originalDTM = getOriginalDataTypeManager();
-			DataType conflictingDt = originalDTM.getDataType(getOriginalCategoryPath(), name);
+
+			DataTypeManager originalDtm = getOriginalDataTypeManager();
+			DataType conflictingDt = originalDtm.getDataType(getOriginalCategoryPath(), name);
 			if (conflictingDt != null) {
 				dialog.setStatusText("A data type named \"" + name + "\" already exists.");
 				return false;
@@ -1303,8 +1294,6 @@ class StructureEditorModel extends CompEditorModel {
 		}
 
 		viewDTM.withTransaction("Unpack Component", () -> {
-			Structure viewStruct = (Structure) viewComposite;
-
 			// Get the field name and comment before removing.
 			String fieldName = currentComp.getFieldName();
 			String comment = currentComp.getComment();
@@ -1347,13 +1336,13 @@ class StructureEditorModel extends CompEditorModel {
 						if (!isPackingEnabled()) {
 							if (dtc.isBitFieldComponent()) {
 								BitFieldDataType bitfield = (BitFieldDataType) dt;
-								viewStruct.insertBitFieldAt(currentOffset + dtc.getOffset(),
+								viewComposite.insertBitFieldAt(currentOffset + dtc.getOffset(),
 									compLength, bitfield.getBitOffset(), bitfield.getBaseDataType(),
 									bitfield.getDeclaredBitSize(), dtc.getFieldName(),
 									dtc.getComment());
 							}
 							else {
-								viewStruct.insertAtOffset(currentOffset + dtc.getOffset(), dt,
+								viewComposite.insertAtOffset(currentOffset + dtc.getOffset(), dt,
 									compLength, dtc.getFieldName(), dtc.getComment());
 							}
 						}
@@ -1380,7 +1369,6 @@ class StructureEditorModel extends CompEditorModel {
 			comp.setComment(comment);
 		});
 		fixSelection();
-		componentEdited();
 		selectionChanged();
 	}
 }
