@@ -4,9 +4,9 @@
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -181,9 +181,8 @@ public final class ReferenceUtils {
 	 * @throws CancelledException if the monitor is cancelled.
 	 * @deprecated use {@link #findDataTypeFieldReferences(Accumulator, FieldMatcher, Program,
 	 * boolean, TaskMonitor)}.
-	 * @Deprecated(since = "10.2")
 	 */
-	@Deprecated
+	@Deprecated(since = "10.2")
 	public static void findDataTypeReferences(Accumulator<LocationReference> accumulator,
 			DataType dataType, String fieldName, Program program, TaskMonitor monitor)
 			throws CancelledException {
@@ -1098,6 +1097,9 @@ public final class ReferenceUtils {
 		Enum enumm = (Enum) dt;
 		List<String> names = getEnumNames(data, enumm);
 		for (String name : names) {
+			if (!enumm.contains(name)) {
+				continue;
+			}
 			long value = enumm.getValue(name);
 			if (matcher.matches(name, (int) value)) {
 				return true;
