@@ -25,6 +25,7 @@ import javax.swing.SwingUtilities;
 import org.junit.Test;
 
 import docking.action.DockingActionIf;
+import ghidra.app.plugin.core.compositeeditor.CompositeEditorProvider;
 import ghidra.app.util.datatype.EmptyCompositeException;
 import ghidra.framework.options.Options;
 import ghidra.program.model.data.*;
@@ -173,9 +174,10 @@ public class StackEditorProvider1Test extends AbstractStackEditorProviderTest {
 
 		Function f = program.getFunctionManager().getFunctionAt(addr("0x200"));
 		assertStackEditorShowing(f);
+		CompositeEditorProvider<?, ?> p = getComponentProvider(CompositeEditorProvider.class);
+		assertNotNull(p);
+		installProvider(p);
 
-		installProvider(stackEditorMgr.getProvider(program, "FUN_00000200"));
-		assertNotNull(provider);
 		model = ((StackEditorProvider) provider).getModel();
 		assertNotNull(model);
 
