@@ -4,9 +4,9 @@
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -16,7 +16,7 @@
 package ghidra.app.decompiler.component;
 
 import java.awt.*;
-import java.awt.event.MouseEvent;
+import java.awt.event.*;
 import java.math.BigInteger;
 import java.util.*;
 import java.util.List;
@@ -42,7 +42,7 @@ import ghidra.app.decompiler.*;
 import ghidra.app.decompiler.component.hover.DecompilerHoverService;
 import ghidra.app.decompiler.component.margin.*;
 import ghidra.app.plugin.core.decompile.DecompilerClipboardProvider;
-import ghidra.app.plugin.core.decompile.actions.FieldBasedSearchLocation;
+import ghidra.app.plugin.core.decompile.actions.DecompilerSearchLocation;
 import ghidra.app.util.viewer.util.ScrollpaneAlignedHorizontalLayout;
 import ghidra.program.model.address.*;
 import ghidra.program.model.listing.Function;
@@ -80,7 +80,7 @@ public class DecompilerPanel extends JPanel implements FieldMouseListener, Field
 
 	private FieldHighlightFactory hlFactory;
 	private ClangHighlightController highlightController;
-	private Map<String, ClangDecompilerHighlighter> highlightersById = new HashMap<>();
+	private Map<String, DecompilerHighlighter> highlightersById = new HashMap<>();
 	private PendingHighlightUpdate pendingHighlightUpdate;
 	private SwingUpdateManager highlighCursorUpdater = new SwingUpdateManager(() -> {
 		if (pendingHighlightUpdate != null) {
@@ -89,6 +89,7 @@ public class DecompilerPanel extends JPanel implements FieldMouseListener, Field
 		}
 	});
 
+	private ActiveMiddleMouse activeMiddleMouse;
 	private int middleMouseHighlightButton;
 	private Color middleMouseHighlightColor;
 	private Color currentVariableHighlightColor;
@@ -126,6 +127,19 @@ public class DecompilerPanel extends JPanel implements FieldMouseListener, Field
 		fieldPanel.addFieldLocationListener(this);
 		fieldPanel.addLayoutListener(this);
 
+		fieldPanel.setName("Decompiler View");
+		fieldPanel.getAccessibleContext().setAccessibleName("Decompiler View");
+
+		fieldPanel.addComponentListener(new ComponentAdapter() {
+			@Override
+			public void componentResized(ComponentEvent e) {
+				for (DecompilerMarginProvider provider : marginProviders) {
+					provider.getComponent().invalidate();
+				}
+				validate();
+			}
+		});
+
 		decompilerHoverProvider = new DecompilerHoverProvider();
 
 		searchHighlightColor = options.getSearchHighlightColor();
@@ -145,6 +159,10 @@ public class DecompilerPanel extends JPanel implements FieldMouseListener, Field
 		}
 	}
 
+	public DecompilerController getController() {
+		return controller;
+	}
+
 	public List<ClangLine> getLines() {
 		return layoutController.getLines();
 	}
@@ -181,13 +199,20 @@ public class DecompilerPanel extends JPanel implements FieldMouseListener, Field
 		return highlightController.getHighlighterHighlights(highligter);
 	}
 
-	private Set<ClangDecompilerHighlighter> getSecondaryHighlihgtersByFunction(Function function) {
+	public TokenHighlights getMiddleMouseHighlights() {
+		if (activeMiddleMouse != null) {
+			return activeMiddleMouse.getHighlights();
+		}
+		return null;
+	}
+
+	private Set<DecompilerHighlighter> getSecondaryHighlihgtersByFunction(Function function) {
 		return highlightController.getSecondaryHighlighters(function);
 	}
 
 	/**
 	 * Removes all secondary highlights for the current function
-	 * 
+	 *
 	 * @param function the function containing the secondary highlights
 	 */
 	public void removeSecondaryHighlights(Function function) {
@@ -199,31 +224,46 @@ public class DecompilerPanel extends JPanel implements FieldMouseListener, Field
 	}
 
 	public void addSecondaryHighlight(ClangToken token) {
-		ColorProvider cp = highlightController.getRandomColorProvider();
+		ColorProvider cp = highlightController.getGeneratedColorProvider();
 		addSecondaryHighlight(token.getText(), cp);
 	}
 
 	public void addSecondaryHighlight(ClangToken token, Color color) {
-		ColorProvider cp = t -> color;
+		ColorProvider cp = new DefaultColorProvider("User Secondary Highlight", color);
 		addSecondaryHighlight(token.getText(), cp);
 	}
 
 	private void addSecondaryHighlight(String tokenText, ColorProvider colorProvider) {
 		NameTokenMatcher matcher = new NameTokenMatcher(tokenText, colorProvider);
-		ClangDecompilerHighlighter highlighter = createHighlighter(matcher);
+		DecompilerHighlighter highlighter = createHighlighter(matcher);
 		applySecondaryHighlights(highlighter);
 	}
 
-	private void applySecondaryHighlights(ClangDecompilerHighlighter highlighter) {
+	private void applySecondaryHighlights(DecompilerHighlighter highlighter) {
 		Function function = decompileData.getFunction();
 		highlightController.addSecondaryHighlighter(function, highlighter);
 		highlighter.applyHighlights();
 	}
 
-	private void togglePrimaryHighlight(FieldLocation location, Field field, Color highlightColor) {
+	private void toggleMiddleMouseHighlight(FieldLocation location, Field field) {
 		ClangToken token = ((ClangTextField) field).getToken(location);
-		Supplier<List<ClangToken>> lazyTokens = () -> findTokensByName(token.getText());
-		highlightController.togglePrimaryHighlights(middleMouseHighlightColor, lazyTokens);
+
+		ActiveMiddleMouse previousMiddleMouse = activeMiddleMouse;
+		activeMiddleMouse = null;
+
+		if (previousMiddleMouse != null) {
+			// middle mousing always clears the last middle-mouse highlight
+			previousMiddleMouse.clear();
+
+			if (previousMiddleMouse.matches(token)) {
+				// middle mousing on the same token clears, but does not create a new highlight
+				return;
+			}
+		}
+
+		ActiveMiddleMouse newMiddleMouse = new ActiveMiddleMouse(token.getText());
+		newMiddleMouse.apply();
+		activeMiddleMouse = newMiddleMouse;
 	}
 
 	void addHighlighterHighlights(ClangDecompilerHighlighter highlighter,
@@ -235,14 +275,14 @@ public class DecompilerPanel extends JPanel implements FieldMouseListener, Field
 		highlightController.removeHighlighterHighlights(highlighter);
 	}
 
-	public ClangDecompilerHighlighter createHighlighter(CTokenHighlightMatcher tm) {
+	public DecompilerHighlighter createHighlighter(CTokenHighlightMatcher tm) {
 		UUID uuId = UUID.randomUUID();
 		String id = uuId.toString();
 		return createHighlighter(id, tm);
 	}
 
-	public ClangDecompilerHighlighter createHighlighter(String id, CTokenHighlightMatcher tm) {
-		ClangDecompilerHighlighter currentHighlighter = highlightersById.get(id);
+	public DecompilerHighlighter createHighlighter(String id, CTokenHighlightMatcher tm) {
+		DecompilerHighlighter currentHighlighter = highlightersById.get(id);
 		if (currentHighlighter != null) {
 			currentHighlighter.dispose();
 		}
@@ -258,7 +298,7 @@ public class DecompilerPanel extends JPanel implements FieldMouseListener, Field
 	}
 
 	void removeHighlighter(String id) {
-		ClangDecompilerHighlighter highlighter = highlightersById.remove(id);
+		DecompilerHighlighter highlighter = highlightersById.remove(id);
 		highlightController.removeHighlighter(highlighter);
 	}
 
@@ -297,40 +337,62 @@ public class DecompilerPanel extends JPanel implements FieldMouseListener, Field
 
 	/**
 	 * This function is used to alert the panel that a token was renamed. If the token being renamed
-	 * had a secondary highlight, we must re-apply the highlight to the new token.
-	 * 
+	 * had a middle-mouse or secondary highlight, we must re-apply the highlights to the new token.
+	 *
 	 * <p>
 	 * This is not needed for highlighter service highlights, since they get called again to
 	 * re-apply highlights. It is up to that highlighter to determine if highlighting still applies
 	 * to the new token name. Alternatively, for secondary highlights, we know the user chose the
 	 * highlight based upon name. Thus, when the name changes, we need to take action to update the
 	 * secondary highlight.
-	 * 
+	 *
 	 * @param token the token being renamed
 	 * @param newName the new name of the token
 	 */
 	public void tokenRenamed(ClangToken token, String newName) {
+		repairMiddleMouseSelectionForRename(token, newName);
+		repairSecondarySelectionForRename(token, newName);
+	}
 
+	private void repairSecondarySelectionForRename(ClangToken token, String newName) {
 		Color hlColor = highlightController.getSecondaryHighlight(token);
 		if (hlColor == null) {
 			return; // not highlighted
 		}
 
-		// remove the old highlighter
 		highlightController.removeSecondaryHighlights(token);
 
+		// Add the new highlighter when we have rebuilt the token
 		controller.doWhenNotBusy(() -> {
 			addSecondaryHighlight(newName, t -> hlColor);
 		});
 	}
 
+	private void repairMiddleMouseSelectionForRename(ClangToken token, String newName) {
+		if (activeMiddleMouse == null || !activeMiddleMouse.matches(token)) {
+			return;
+		}
+
+		activeMiddleMouse.clear();
+		activeMiddleMouse = new ActiveMiddleMouse(newName);
+
+		// Apply the new middle-mouse highlighter when we have rebuilt the token
+		controller.doWhenNotBusy(() -> {
+			activeMiddleMouse.apply();
+		});
+	}
+
 	private void cloneGlobalHighlighters(DecompilerPanel sourcePanel) {
 
-		Set<ClangDecompilerHighlighter> globalHighlighters =
+		Set<DecompilerHighlighter> globalHighlighters =
 			sourcePanel.highlightController.getGlobalHighlighters();
-		for (ClangDecompilerHighlighter otherHighlighter : globalHighlighters) {
+		for (DecompilerHighlighter otherHighlighter : globalHighlighters) {
 
-			ClangDecompilerHighlighter newHighlighter = otherHighlighter.clone(this);
+			if (!(otherHighlighter instanceof ClangDecompilerHighlighter clangHighlighter)) {
+				continue;
+			}
+
+			DecompilerHighlighter newHighlighter = clangHighlighter.clone(this);
 			highlightersById.put(newHighlighter.getId(), newHighlighter);
 
 			TokenHighlights otherHighlighterTokens =
@@ -349,7 +411,7 @@ public class DecompilerPanel extends JPanel implements FieldMouseListener, Field
 	/**
 	 * Called by the provider to clone all highlights in the source panel and apply them to this
 	 * panel
-	 * 
+	 *
 	 * @param sourcePanel the panel that was cloned
 	 */
 	public void cloneHighlights(DecompilerPanel sourcePanel) {
@@ -361,15 +423,20 @@ public class DecompilerPanel extends JPanel implements FieldMouseListener, Field
 		// clone will match the cloned decompiler.
 		//
 		Function function = decompileData.getFunction();
-		Set<ClangDecompilerHighlighter> secondaryHighlighters =
+		Set<DecompilerHighlighter> secondaryHighlighters =
 			sourcePanel.getSecondaryHighlihgtersByFunction(function);
 
 		//
 		// We do NOT clone the secondary highlighters.  This allows the user the remove them
 		// from the primary provider without effecting the cloned provider and vice versa.
 		//
-		for (ClangDecompilerHighlighter highlighter : secondaryHighlighters) {
-			ClangDecompilerHighlighter newHighlighter = highlighter.copy(this);
+		for (DecompilerHighlighter highlighter : secondaryHighlighters) {
+
+			if (!(highlighter instanceof ClangDecompilerHighlighter clangHighlighter)) {
+				continue;
+			}
+
+			DecompilerHighlighter newHighlighter = clangHighlighter.copy(this);
 			highlightersById.put(newHighlighter.getId(), newHighlighter);
 			applySecondaryHighlights(newHighlighter);
 		}
@@ -393,7 +460,7 @@ public class DecompilerPanel extends JPanel implements FieldMouseListener, Field
 
 	/**
 	 * This function sets the current window display based on our display state
-	 * 
+	 *
 	 * @param decompileData the new data
 	 */
 	void setDecompileData(DecompileData decompileData) {
@@ -442,9 +509,8 @@ public class DecompilerPanel extends JPanel implements FieldMouseListener, Field
 			return;
 		}
 
-		Set<ClangDecompilerHighlighter> globalHighlighters =
-			highlightController.getGlobalHighlighters();
-		for (ClangDecompilerHighlighter highlighter : globalHighlighters) {
+		Set<DecompilerHighlighter> globalHighlighters = highlightController.getGlobalHighlighters();
+		for (DecompilerHighlighter highlighter : globalHighlighters) {
 			highlighter.clearHighlights();
 			highlighter.applyHighlights();
 		}
@@ -457,9 +523,9 @@ public class DecompilerPanel extends JPanel implements FieldMouseListener, Field
 			return;
 		}
 
-		Set<ClangDecompilerHighlighter> secondaryHighlighters =
+		Set<DecompilerHighlighter> secondaryHighlighters =
 			getSecondaryHighlihgtersByFunction(function);
-		for (ClangDecompilerHighlighter highlighter : secondaryHighlighters) {
+		for (DecompilerHighlighter highlighter : secondaryHighlighters) {
 			highlighter.clearHighlights();
 			highlighter.applyHighlights();
 		}
@@ -565,7 +631,7 @@ public class DecompilerPanel extends JPanel implements FieldMouseListener, Field
 
 	/**
 	 * Put cursor on first token in the list
-	 * 
+	 *
 	 * @param tokens the tokens to search for
 	 */
 	private void goToBeginningOfLine(List<ClangToken> tokens) {
@@ -677,7 +743,7 @@ public class DecompilerPanel extends JPanel implements FieldMouseListener, Field
 	/**
 	 * Passing false signals to disallow navigating to new functions from within the panel by using
 	 * the mouse.
-	 * 
+	 *
 	 * @param enabled false disabled mouse function navigation
 	 */
 	void setMouseNavigationEnabled(boolean enabled) {
@@ -706,7 +772,7 @@ public class DecompilerPanel extends JPanel implements FieldMouseListener, Field
 		}
 
 		if (buttonState == middleMouseHighlightButton && clickCount == 1) {
-			togglePrimaryHighlight(location, field, middleMouseHighlightColor);
+			toggleMiddleMouseHighlight(location, field);
 		}
 	}
 
@@ -757,19 +823,6 @@ public class DecompilerPanel extends JPanel implements FieldMouseListener, Field
 			controller.goToFunction(function, newWindow);
 			return;
 		}
-
-		// TODO no idea what this is supposed to be handling...someone doc this please
-//		String labelName = functionToken.getText();
-//		if (labelName.startsWith("func_0x")) {
-//			try {
-//				Address addr =
-//					decompileData.getFunction().getEntryPoint().getAddress(labelName.substring(7));
-//				controller.goToAddress(addr, newWindow);
-//			}
-//			catch (AddressFormatException e) {
-//				controller.goToLabel(labelName, newWindow);
-//			}
-//		}
 	}
 
 	private void tryGoToLabel(ClangLabelToken token, boolean newWindow) {
@@ -953,33 +1006,15 @@ public class DecompilerPanel extends JPanel implements FieldMouseListener, Field
 			location.getIndex().intValue(), location.col);
 	}
 
-//==================================================================================================
-// Search Methods
-//==================================================================================================
-
-	public SearchLocation searchText(String text, FieldLocation startLocation,
-			boolean forwardDirection) {
-		return layoutController.findNextTokenForSearch(text, startLocation, forwardDirection);
-	}
-
-	public SearchLocation searchTextRegex(String text, FieldLocation startLocation,
-			boolean forwardDirection) {
-		return layoutController.findNextTokenForSearchRegex(text, startLocation, forwardDirection);
-	}
-
 	public void setSearchResults(SearchLocation searchLocation) {
 		currentSearchLocation = searchLocation;
 		repaint();
 	}
 
-	public FieldBasedSearchLocation getSearchResults() {
-		return (FieldBasedSearchLocation) currentSearchLocation;
+	public DecompilerSearchLocation getSearchResults() {
+		return (DecompilerSearchLocation) currentSearchLocation;
 	}
 
-//==================================================================================================
-// End Search Methods
-//==================================================================================================
-
 	public Color getCurrentVariableHighlightColor() {
 		return currentVariableHighlightColor;
 	}
@@ -991,7 +1026,7 @@ public class DecompilerPanel extends JPanel implements FieldMouseListener, Field
 	/**
 	 * The color used in a primary highlight to mark the token that was clicked. This is used in
 	 * 'slice' actions to mark the source of the slice.
-	 * 
+	 *
 	 * @return the color
 	 */
 	public Color getSpecialHighlightColor() {
@@ -1053,7 +1088,7 @@ public class DecompilerPanel extends JPanel implements FieldMouseListener, Field
 
 	/**
 	 * Returns a single selected token; null if there is no selection or multiple tokens selected.
-	 * 
+	 *
 	 * @return a single selected token; null if there is no selection or multiple tokens selected.
 	 */
 	public ClangToken getSelectedToken() {
@@ -1083,10 +1118,10 @@ public class DecompilerPanel extends JPanel implements FieldMouseListener, Field
 
 	/**
 	 * Get the line number for the given y position, relative to the scroll panel
-	 * 
+	 *
 	 * <p>
 	 * If the y position is below all the lines, the last line is returned.
-	 * 
+	 *
 	 * @param y the y position
 	 * @return the line number, or 0 if not applicable
 	 */
@@ -1201,6 +1236,18 @@ public class DecompilerPanel extends JPanel implements FieldMouseListener, Field
 		buildPanels();
 	}
 
+	@Override
+	public synchronized void addFocusListener(FocusListener l) {
+		// we are not focusable, defer to contained field panel
+		fieldPanel.addFocusListener(l);
+	}
+
+	@Override
+	public synchronized void removeFocusListener(FocusListener l) {
+		// we are not focusable, defer to contained field panel
+		fieldPanel.removeFocusListener(l);
+	}
+
 	private void buildPanels() {
 		removeAll();
 		add(buildLeftComponent(), BorderLayout.WEST);
@@ -1232,7 +1279,7 @@ public class DecompilerPanel extends JPanel implements FieldMouseListener, Field
 			int highlightLine = cField.getLineNumber();
 
 			FieldLocation searchCursorLocation =
-				((FieldBasedSearchLocation) currentSearchLocation).getFieldLocation();
+				((DecompilerSearchLocation) currentSearchLocation).getFieldLocation();
 			int searchLineNumber = searchCursorLocation.getIndex().intValue() + 1;
 			if (highlightLine != searchLineNumber) {
 				// only highlight the match on the actual line
@@ -1297,7 +1344,15 @@ public class DecompilerPanel extends JPanel implements FieldMouseListener, Field
 	private class DecompilerFieldPanel extends FieldPanel {
 
 		public DecompilerFieldPanel(LayoutModel model) {
-			super(model);
+			super(model, "Decompiler");
+			// In the decompiler each field represents a line, so make the field description
+			// simply be the line number
+			setFieldDescriptionProvider((l, f) -> {
+				if (f == null) {
+					return null;
+				}
+				return "line " + (l.getIndex().intValue() + 1);
+			});
 		}
 
 		/**
@@ -1305,7 +1360,7 @@ public class DecompilerPanel extends JPanel implements FieldMouseListener, Field
 		 * an event to the rest of the tool. (This is in contrast to a field panel
 		 * <code>goTo</code>, which we use to simply move the cursor, but not trigger an tool-level
 		 * navigation event.)
-		 * 
+		 *
 		 * @param lineNumber the line number
 		 * @param column the column within the line
 		 */
@@ -1344,4 +1399,56 @@ public class DecompilerPanel extends JPanel implements FieldMouseListener, Field
 			}
 		}
 	}
+
+	private class MiddleMouseColorProvider implements ColorProvider {
+
+		@Override
+		public Color getColor(ClangToken token) {
+			return middleMouseHighlightColor;
+		}
+
+		@Override
+		public String toString() {
+			return "Middle Mouse Color Provider " + middleMouseHighlightColor;
+		}
+	}
+
+	/**
+	 * A class to track the current middle moused token.
+	 */
+	private class ActiveMiddleMouse {
+
+		private String tokenText;
+		private DecompilerHighlighter highlighter;
+
+		ActiveMiddleMouse(String tokenText) {
+			this.tokenText = tokenText;
+
+			ColorProvider cp = new MiddleMouseColorProvider();
+			NameTokenMatcher matcher = new NameTokenMatcher(tokenText, cp);
+			this.highlighter = createHighlighter(matcher);
+		}
+
+		TokenHighlights getHighlights() {
+			return highlightController.getHighlighterHighlights(highlighter);
+		}
+
+		boolean matches(ClangToken other) {
+			return tokenText.equals(other.getText());
+		}
+
+		void clear() {
+			highlightController.removeHighlighter(highlighter);
+		}
+
+		void apply() {
+			applySecondaryHighlights(highlighter);
+		}
+
+		@Override
+		public String toString() {
+			return "Middle Mouse Token " + tokenText;
+		}
+	}
+
 }
